// ===============================================================
// =========== PRIMA CERCHIO ORARIO POI ANTIORARIO ===============
// ===============================================================

#include <OrangutanAnalog.h>
#include <OrangutanMotors.h>
OrangutanAnalog analog;
OrangutanMotors motors;

const int Num = 4;  // Numero letture ADC per media

// ========================================================================
// ============================= PIN SENSORI ==============================
// ========================================================================
const int sharpL_pin = 0;   // SHARP SINISTRO
const int sharpR_pin = 1;   // SHARP DESTRO
const int battery_pin = 2;  // Batteria
const int cny70L_pin = 3;   // CNY70 SINISTRO
const int cny70R_pin = 5;   // CNY70 DESTRO

float sVL;
float sVR;


const int B = 0;   // Bianco
const int N = 1;   // Nero
const int L = -1;  // Left
const int R = 1;   // Right

const int Vmax0 = 100; //100
const int Vmed0 = 55; //50
const int Vmin0 = 35; //50
int Vmax = Vmax0;
int Vmed = Vmed0;
int Vmin = Vmin0;
int VL = Vmed0;
int VR = Vmed0;

const int dV = 0; //-10
const int dist = 100;   // Distanza minima
const int max_battery = 800; // Valore letto dall'ADC a batterie nuove
const int soglia = 300;   // soglia B-N

int val_cny70L = B;
int val_cny70R = B;
int val_cny70L_pre = B;
int val_cny70R_pre = B;
int Aval_cny70L = 0;
int Aval_cny70R = 0;
int val_sharpL = 0;
int val_sharpR = 0;
int val_battery = 0;

int dir = 0;

unsigned long now, nextBatt, nextMap;

float Zk = 0.3;

boolean cir = true;

void setup() {                
  analog.setMode(MODE_10_BIT);    // 10-bit analog-to-digital conversions
  nextBatt = millis();
  int pot = analog.readTrimpot();    // determine the trimpot position
  if (pot >=255) 
  {
    sVL=1+(pot/100);
    sVR=1;
  }
  else
  {
    sVL=1;
    sVR=1+(pot/100);
  }
}

void loop()
{
  // ========================================================================
  // ======================= CONTROLLO CARICA BATTERIA ======================
  // ========================================================================
  now = millis();
  if (now > nextBatt) {
    nextBatt = now + 60000;

    val_battery = 0;
    for (int j = 0; j < Num; j++) {
      val_battery += analog.read(battery_pin);
      delayMicroseconds(100);
    }
    val_battery /= Num;  

    Vmin = float(Vmin0) * float(max_battery) / float(val_battery);
    Vmed = float(Vmed0) * float(max_battery) / float(val_battery);
    Vmax = float(Vmax0) * float(max_battery) / float(val_battery);
  }

  // Mappa Logistica
  now = millis();
  if (now > nextMap) {
    nextMap = now + 20000;

    Zk = 4.0*Zk*(1 - Zk);
    //=================================================
    //============ FA UNA SERIE DI CURVE ==============
    //=================================================
    if (cir) {
      VL = Vmed;
      VR = Vmin;
    }
    else {
      VL = Vmin;
      VR = Vmed;
    }
    cir = !cir;
  }

  motors.setSpeeds(VL*sVL, VR*sVR);  // set speeds of motors 1 (SX) and 2 (DX)


  // ========================================================================
  // ===================== LETTURA VALORI DEI SENSORI =======================
  // ========================================================================  val_sharpL = val_sharpR = Aval_cny70L = Aval_cny70R = 0;
  for (int j = 0; j < Num; j++) {
    val_sharpL += analog.read(sharpL_pin);
    delayMicroseconds(100);
    val_sharpR += analog.read(sharpR_pin);
    delayMicroseconds(100);
    Aval_cny70L += analog.read(cny70L_pin);
    delayMicroseconds(100);
    Aval_cny70R += analog.read(cny70R_pin);
    delayMicroseconds(100);
  }
  val_sharpL /= Num;
  val_sharpR /= Num;
  Aval_cny70L /= Num;
  Aval_cny70R /= Num;

  // ========================================================================
  // ==========  conversione soglia bianco/vero cny70 su analogico ==========
  // ========================================================================
  if (Aval_cny70L > soglia){
    val_cny70L=B;
  }
  else {
    val_cny70L=N;
  }
  if (Aval_cny70R > soglia){
    val_cny70R=B;
  }
  else {
    val_cny70R=N;
  }

  delay(1);

}



